/*******************************************************************************
 *
 *  HomeAutomation_GUI.pde 
 *           - PC demo application for establishing a UDP Wifi socket
 *             with the MSP-EXP430FR5739 and CC3000 Experimenter's Boards.
 * 
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the project's author nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/******************************************************************************
 *       MSP-EXP430FR5739 and CC3000 Home Automation Example GUI
 * Desc:
 * This PC GUI application communicates with the MSP-EXP430FR5739 via the CC3000  
 * module to send receive regular temperature updates and send temperature 
 * setpoints and dimmer values for 2 LEDs. 
 * 
 * to communicate to the CC3000 we use 3 UDP sockets and use multicast addresss
 * We use multicast because we don't know what the IP address is of the CC3000
 * Nor does it know ours.  Furthermore, multicast is useful if we wanted multiple
 * listeners.  The two sockets used for transmitting the Light and Thermonstat 
 * data are updated whenever there is a user activated update or at least every
 * 3 seconds.  One could lower this rate but it must be < 55 seconds in order 
 * for the CC3000 to keep a socket open. 
 * 
 *
 * M. Schneider
 * PDE, Ltd.
 * Ver 1.00 September 2011
 ******************************************************************************/ 
// import UDP library
import hypermedia.net.*;

/*
* UDP multicast address and port definitions
*/
UDP udp, udpTx;  // define the UDP object
String MyIpAddress;
String MULTICAST_IP       = "224.0.0.1";	// the remote IP address (multicast)
int LIGHT_PORT = 0xC738;
int THERMOSTAT_PORT_TX = 0xC739;
int THERMOSTAT_PORT_RX = 0xC73A;

/*
* set the number of Lights objects
*/
int NUM_OF_LIGHTS = 2;

/*
* main objects
*/
classThermostat Thermostat;
classLights[] Lights;
classTwitterCloud Twitter;

/*
* misc
*/
String TI_LINK = "http://www.ti.com"; 
String PDE_LINK = "http://www.pde.com"; 
PImage PDEImg, TIImg;
int TI_IMG_X = 840; 
int TI_IMG_Y = 573;
int PDE_IMG_X = 29;
int PDE_IMG_Y = 612;
PFont f;
String TemperatureUnitsString;
boolean UpdateSetPointOkFlag;
String[] TwitterAcc = new String[5];

/*
* Variables to keep time
*/
int PreviousSeconds, CurrentSeconds, PreviousMinutes, CurrentMinutes;
int[] SecondsPassedBeforeLightsUpdate = new int[NUM_OF_LIGHTS];
int   SecondsPassedBeforeThermUpdate;

/*
* Constants to define the minimum update rate from the PC in the event there are now 
* updates to send to the MSP430.  If these are > 55 seconds than the CC3000 will close 
* the UDP sockets due to inactivity.
*/
int MIN_SECONDS_LIGHTS_UPDATE    =     3;
int MIN_SECONDS_THERM_UPDATE     =     3;

/**
 *  Define the GUI layout
 */ 
 // Thermostat
int THERM_X_LOC  = 63;
int THERM_Y_LOC  = 47;
 // Lights
int LIGHT_X_LOC_START = 233;
int LIGHT_Y_LOC_START = 47;
int LIGHT_X_LOC_STEP  = 369;
int LIGHT_Y_LOC_STEP  = 0;
// Twitter Cloud
int CLOUD_X_LOC = 331;
int CLOUD_Y_LOC = 361;
 
/**
 * init
 */
void setup() {  
  size(952, 681); 
  // reduce frame Rate from 60 to 45 fps for older machines
  frameRate(45);
  smooth();
  
  // Open Configuration file for Temperature units and Twitter account
  // information
  ReadConfigFile();
  
  // create a new datagram connection on port THERMOSTAT_PORT
  // and wait for incomming messages  
  udp = new UDP( this, THERMOSTAT_PORT_RX, MULTICAST_IP);  
  udp.log( true ); 		// <-- printout the connection activity  
  udp.listen( true );  
  
  // If the user didn't configure an IP address then just use the 
  // mutlicast address.  the UDP stack will then choose the default
  // network adapter.
  if(MyIpAddress.length() == 0)
  {
    MyIpAddress = MULTICAST_IP;
  }
  println("add:"+ MyIpAddress);
  udpTx = new UDP( this, LIGHT_PORT, MyIpAddress); 
  println(udp.isJoined());
  println(udp.isMulticast());
  
  f = loadFont("ComicSansMS-48.vlw"); 
  
  // Setup the Thermostat
  Thermostat = new classThermostat(THERM_X_LOC,THERM_Y_LOC);  
  if(TemperatureUnitsString.equals("C"))
  {
    Thermostat.SetUnits(Thermostat.UNITS_C);
  }
  else if(TemperatureUnitsString.equals("F"))
  {
    Thermostat.SetUnits(Thermostat.UNITS_F);
  }
  Thermostat.setup();
  SecondsPassedBeforeThermUpdate = MIN_SECONDS_THERM_UPDATE;
  
  // Setup the Lights
  Lights = new classLights[NUM_OF_LIGHTS];
  for(int i = 0; i < NUM_OF_LIGHTS; i++)
  {
    Lights[i] = new classLights(LIGHT_X_LOC_START + (i*LIGHT_X_LOC_STEP), LIGHT_Y_LOC_START+ (i*LIGHT_Y_LOC_STEP));
    Lights[i].setup(); 
    SecondsPassedBeforeLightsUpdate[i] = MIN_SECONDS_LIGHTS_UPDATE;
  }
  
  // Setup Twitter
  Twitter = new classTwitterCloud(CLOUD_X_LOC, CLOUD_Y_LOC, TwitterAcc);
  Twitter.setup();
  
  // Logos
  PDEImg = loadImage("PDE_Logo.jpg");
  TIImg = loadImage("TI_Logo.jpg");
  
  // Reset time
  CurrentSeconds = PreviousSeconds = 0;
}

// Main App Draw loop
void draw() {
  background(0x262626);
  Thermostat.draw();
  CurrentSeconds = second();  
  CurrentMinutes = minute();  
  
  /* Only check Twitter 1/min because twitter rate limits.
     Currently, at the time of this writting 
     Unauthenticated calls are permitted 150 requests per hour */  
  if(CurrentMinutes != PreviousMinutes)
  {
    PreviousMinutes = CurrentMinutes;
    String TwitterStatus;
    TwitterStatus = Twitter.CheckForTwitterMessage();
    if(TwitterStatus != null)
    {
      ParseIncomingMessage(TwitterStatus);      
    }
  }
  // Counter for forcing of minimum rate Light Updates
  if(CurrentSeconds != PreviousSeconds)
  {
    PreviousSeconds = CurrentSeconds; 
    UpdateSetPointOkFlag = true;
    for(int i = 0; i < NUM_OF_LIGHTS; i++)
    {
      SecondsPassedBeforeLightsUpdate[i]--;
    }
    SecondsPassedBeforeThermUpdate--;
  }  
  // Counter for forcing of minimum rate Thermostat Updates
  if((Thermostat.SetPointUpdated && UpdateSetPointOkFlag == true) ||
     SecondsPassedBeforeThermUpdate == 0)
  {
    UpdateSetPointOkFlag = false;
    Thermostat.SetPointUpdated = false;     
    SendThermSetPoint(int(Thermostat.SetPoint));
    SecondsPassedBeforeThermUpdate = MIN_SECONDS_THERM_UPDATE;
  }
  
  // Draw Lights
  for(int i = 0; i < NUM_OF_LIGHTS; i++)
  {
    Lights[i].draw();    
    if(Lights[i].DimmerUpdated || (SecondsPassedBeforeLightsUpdate[i] == 0))
    {
      Lights[i].DimmerUpdated = false;
      SendLightData(i, Lights[i].Dimmer);
      // Reset Timer
      SecondsPassedBeforeLightsUpdate[i] = MIN_SECONDS_LIGHTS_UPDATE;      
    }
  
  }
  
  // Draw Twitter cloud if active
  Twitter.draw();
  // Draw the logos
  DrawLogos();
}

/*
Parses Text string from Twitter status message.
Format is:
T,xx	-> Set temperature to xx in °C or °F  [eg. T,25 ] set temperature to 25°C
L,x,y	-> set Light(x) to y state.  [eg. l,4,1 ] turn LED 4 on
S	-> Request a status update.  

Case sensitive and format is strict
*/
void ParseIncomingMessage(String TwitterStatus){
  String TwitterMessage;
  if(TwitterStatus.charAt(0) == 'T' && TwitterStatus.charAt(1) == ',')
  {
      float temp = float(TwitterStatus.substring(2,TwitterStatus.length()));
      try  
       {  
          if(temp > 0)
          {
            TwitterStatus = "set temp to: " + temp; 
            Thermostat.setSetPointAndLimits(temp);
          }
          else
          {
            TwitterStatus = null;
          }
       }  
       catch( Exception e)  
       {  
         TwitterStatus = null;
       }        
  }
  else if(TwitterStatus.charAt(0) == 'L' && TwitterStatus.charAt(1) == ',' && TwitterStatus.charAt(3) == ',')
  {
      int idx = int(TwitterStatus.charAt(2)) - '0';
      int state = int(TwitterStatus.charAt(4))- '0';
      if( (idx >= 0) && (idx <= NUM_OF_LIGHTS) && (state >= 0) &&  (state <= 1) )
      {
        if(state == 0)
        {
          TwitterStatus = "Set Light[" + idx + "] = Off";          
        }
        else
        {
          TwitterStatus = "Set Light[" + idx + "] = On";
        }        
        Lights[idx].SetDimmerPosition(boolean(state));
      }
      else
      {
        TwitterStatus = null;
      }
  }
  else if(TwitterStatus.substring(0,6).equals("Status"))
  {
    String[] strLightState = new String[NUM_OF_LIGHTS];    
    for(int i = 0; i < NUM_OF_LIGHTS; i++)
    {
      if(Lights[i].Dimmer > 8)
      {
        strLightState[i] = "On";
      }
      else 
      {
        strLightState[i] = "Off";
      }
    }
    Twitter.UpdateStatus("Current Temperature is " + Thermostat.temperature + ". Light #0 is " + strLightState[0] + ". Light #1 is " + strLightState[1]);
    TwitterStatus = "Status Request serviced";
  }
  else
  {
    TwitterStatus = null;
  }
  
  print(TwitterStatus);
  if(TwitterStatus != null)
  {
    Twitter.PrintTwitterMessage(TwitterStatus);
  }
}
      
/*
 * Transmitts the Dimmer Light values to the MSP430 using
 * MULTICAST_IP and LIGHT_PORT.
*/
void SendLightData(int LightId, int Dimmer)
{
  byte[] buffer = new byte[7];
  // Start
  buffer[0] = 'S';
  buffer[1] = '1';
  // Data
  buffer[2] = byte(Dimmer);
  buffer[3] = byte(Dimmer>>8);
  buffer[4] = byte(LightId);
  // END
  buffer[5] = 'S';
  buffer[6] = '2';
  
  String message = new String(buffer);  
  int port        = LIGHT_PORT;         // the destination port
     
  udpTx.send( message, MULTICAST_IP, port );
}

/*
 * Transmitts the Thermostat Set and alarm points to the MSP430 using
 * MULTICAST_IP and THERMOSTAT_PORT_TX.
*/
void SendThermSetPoint(int Setpoint)
{
    byte[] buffer = new byte[11];
    // Start
    buffer[0] = 'S';
    buffer[1] = '1';
    // Data
    buffer[2] = 0;
    buffer[3] = byte(Thermostat.ADCSetPoint);
    buffer[4] = byte(Thermostat.ADCSetPoint>>8);
    buffer[5] = byte(Thermostat.ADCLimitLow);
    buffer[6] = byte(Thermostat.ADCLimitLow>>8);
    buffer[7] = byte(Thermostat.ADCLimitHigh);
    buffer[8] = byte(Thermostat.ADCLimitHigh>>8);
    // End
    buffer[9] = 'S';
    buffer[10] = '2';
    
    String message = new String(buffer);    
    int port        = THERMOSTAT_PORT_TX; // the destination port
       
    udpTx.send( message, MULTICAST_IP, port );
}

/**
 * To perform any action on datagram reception, you need to implement this 
 * handler in your code. This method will be automatically called by the UDP 
 * object each time he receive a nonnull message.
 * By default, this method have just one argument (the received message as 
 * byte[] array), but in addition, two arguments (representing in order the 
 * sender IP address and his port) can be set like below.
 */
// void receive( byte[] data ) { 			// <-- default handler
void receive( byte[] data, String ip, int port ) {	// <-- extended handler
  // print the result
  int tempInt = int(data[2]<<8) + int(data[1]);
  int state = data[3];
  println(state);
  if(data[0] == 0x03)
  {
    Thermostat.temperature = Thermostat.ConvertToTemperature(float(tempInt));
    if(Thermostat.state != state)
    {
      Thermostat.state = state;
      // update State and send Twitter alarm message      
      if(Thermostat.state == Thermostat.THERMOSTAT_STATE_ALARM)
      {
        Twitter.UpdateStatus("Temperature Alarm.  Temperature is out of range: "+ Thermostat.temperature + TemperatureUnitsString);
        Twitter.PrintTwitterMessage("Temperature Alarm");
      }
      else if(Thermostat.state == Thermostat.THERMOSTAT_STATE_IDLE)
      {
        Twitter.UpdateStatus("Temperature is now OK: "+ Thermostat.temperature + TemperatureUnitsString);
        Twitter.PrintTwitterMessage("Temperature is now OK");
      }            
    }
  }
}

/*
 * Reads an XML file and parses the Temperature units and the Twitter user account info.
 */
void ReadConfigFile () {
    XMLElement xml = new XMLElement(this, "../manifest.xml");
    XMLElement[] xmlc = xml.getChildren();
    
    XMLElement Units = xmlc[0].getChild(0);
    TemperatureUnitsString = Units.getString("type");
    
    XMLElement TwitterAccount = xmlc[1].getChild(0);     
    TwitterAcc[0] = TwitterAccount.getString("Key");
    TwitterAcc[1] = TwitterAccount.getString("KeySecret");
    TwitterAcc[2] = TwitterAccount.getString("Token");
    TwitterAcc[3] = TwitterAccount.getString("TokenSecret");
    
    XMLElement TwitterUser = xmlc[1].getChild(1);     
    TwitterAcc[4] = TwitterUser.getString("Name");
    
    XMLElement IpAddress = xmlc[2].getChild(0);     
    MyIpAddress = IpAddress.getString("ip");
}

void DrawLogos() {
  // Draw both Logos
  image(PDEImg, PDE_IMG_X, PDE_IMG_Y); 
  image(TIImg, TI_IMG_X, TI_IMG_Y); 
 
  // See if mouse over logo
  if( (mouseY > (TI_IMG_Y)) &&
      (mouseX > (TI_IMG_X)) &&
      (mouseX < (TIImg.width + TI_IMG_X))  )
      {
        // Display link
        fill(30);
        rect(TI_IMG_X-40, TI_IMG_Y+TIImg.height, 150,30);
        fill(255);
        text(TI_LINK, TI_IMG_X-30, TI_IMG_Y+TIImg.height+ 20);
        // if pressed too then open link
        if (mousePressed == true) 
        {
          link(TI_LINK);
          while(mousePressed == true)
          {
            noLoop();
          }
          loop();
        }
      }
  if( (mouseY > (PDE_IMG_Y)) &&
      (mouseX > (PDE_IMG_X)) &&
      (mouseX < (PDEImg.width + PDE_IMG_X))  )
      {
        // Display link
        fill(30);
        rect(PDE_IMG_X, PDE_IMG_Y + PDEImg.height, 170,30);
        fill(255);
        text(PDE_LINK, PDE_IMG_X+10, PDE_IMG_Y + PDEImg.height+ 20);
        // if pressed too then open link
        if (mousePressed == true) 
        {
          link(PDE_LINK);
          while(mousePressed == true)
          {
            noLoop();
          }
          loop();
        }
      }
}
